/* lunar_decls.h
 *
 *   LUNAR - Lightweight Underlay Network Ad hoc Routing
 *
 *   LUNAR methods and variables declaration
 *
 * Copyright (C) 2001-2005 <christian.tschudin@unibas.ch>
 *
 * This file is distributed under the terms of the GNU general Public
 * License (GPL), see the file COPYING
 */


// ----------------------------------------------------------------------
#ifndef LUNAR_DECL_H
#define LUNAR_DECL_H


//LUNAR_DECLARATIONS

#ifndef NO_DEBUG
NS_STATIC void debug_target(char *s, struct lunartarget_s *target);
#endif
NS_STATIC char* lunar_eth2str(char *data);
NS_STATIC char* lunar_ip2str(int ip); // network byte order
#ifdef LUNAR_HAS_IPV6
NS_STATIC char* lunar_ip62str(struct in6_addr *ip6);
NS_EXTERN int prefix_match_len(void *addr1, void *addr2);
NS_EXTERN char addr_scope(struct in6_addr *addr);
NS_EXTERN int IamIPv6Ready(void);
#endif
NS_STATIC char* lunar_hit2str(hit_t *hit);

NS_STATIC  int param2target(struct lunartarget_s *t, xrp_param_t *p);
NS_EXTERN void lunar_rreq_handler(sapfpkt *p, void *aux);
NS_EXTERN void lunar_rreq_handler_uc(int ifnr, struct sockaddr *eth,
		struct xrp_sel_eth_s *reply, struct xrp_sel_eth_s *backwd,
		struct xrp_sel_s *fwd, struct lunartarget_s *target,
		struct lunartarget_s *srchit, struct lunartarget_s *srcaddr,
		struct lunartarget_s *srcname);
NS_EXTERN void lunar_rreq_handler_fwd(sapfpkt *p,
		struct xrp_sel_eth_s *reply, struct xrp_sel_eth_s *backwd,
		struct sockaddr *eth);
NS_EXTERN void lunar_rreq_handler_bc(struct lunartarget_s *target,
				     struct xrp_sel_s *series,
				     struct xrp_sel_eth_s *reply,
				     struct xrp_sel_s *fwd,
				     sapfpkt *p);
NS_STATIC void path_timeout(void *n);
NS_STATIC void start_rreq(int peerndx, int ndx, int udp_port, int trans_id, char namefamily);


NS_STATIC void L3deliver(sapfpkt *pkt, void *aux);

NS_STATIC int namedb_get(int peerndx, char targetfamily, char scope);
NS_STATIC struct lunartarget_s* namedb_getname(int peerndx,
					       char targetfamily, char scope);
NS_STATIC int namedb_getallipnames(int peerndx, int ndx, 
				   struct lunartarget_s *addr);
NS_STATIC int namedb_find(struct lunartarget_s *target);
NS_STATIC int namedb_add(struct lunartarget_s *target);
NS_STATIC void NS_CLASS namedb_delpeer(int ndx);
NS_STATIC void namedb_touch(int ndx);


NS_STATIC int  peer_getindex(struct lunartarget_s *target);
NS_STATIC int  peer_install(struct lunartarget_s *hit,
		    struct lunartarget_s *addr, struct lunartarget_s *name);
NS_STATIC int  peer_getnew(void);
// NS_STATIC int  peer_hasroute(struct lunartarget_s *target);
NS_STATIC void peer_set(int ndx, struct lunartarget_s *target);
NS_STATIC void peer_delpath(struct peer_s *n);
NS_STATIC void peer_del(struct peer_s *n);
NS_STATIC int  path_install(int ndx, sapfsel_t ptr, int plen,
			    char timeroffs);
NS_STATIC int  path_getnew(void);
NS_STATIC void peer_timeout(void *vp);

NS_STATIC struct sapfdest_s*  mkSapfDest(unsigned char *eth, sapfsel_t sel);
NS_STATIC sapfsel_t  mkNetPointer(unsigned char *eth, sapfsel_t sel);
NS_STATIC sapfsel_t  mkHandlerPointer(sapfcallback_t fct, void *aux);
NS_STATIC void  lunar_neigh_map(int ndx);

NS_STATIC void  unicast_rrep_handler(sapfpkt *p, void *ups_);
	
NS_STATIC void unicast_transit_rrep_handler(sapfpkt *p,	
					  void *tr_);
NS_STATIC void  bcast_rep_handler(sapfpkt *p,void *bps_);
	
NS_STATIC void  send_rreq(int radius, sapfsel_t series,
			  struct lunartarget_s *target,
			  sapfsel_t reply, sapfsel_t backsel,
			  struct lunartarget_s *srchit,
			  struct lunartarget_s *srcaddr,
			  struct lunartarget_s *srcname,
			  sapfsel_t fwd);

NS_STATIC int  lunar_do_ndisc(struct lunartarget_s *target,
			      int udp_port, int trans_id, char namefamily);
NS_STATIC void  lunar_do_bc_setup(sapfsel_t *sel, sapfpkt *pkt);
NS_STATIC void  L2sendto(sapfpkt *pkt, void *aux);

NS_STATIC void schedule_rreq(void *data, int len, unsigned long usec);

#ifdef LUNAR_HAS_DNS
NS_EXTERN int print_flags(char flags, char *buf);
NS_EXTERN void lunar_dns_init(void);
NS_EXTERN void lunar_dns_fini(void);
NS_STATIC int  lunar_do_dns(sapfpkt * pkt);
NS_STATIC void send_dns_reply(struct lunartarget_s *query,
			      struct lunartarget_s *reply,
			      int udp_port, int query_id, int ptrstyle);
NS_STATIC void send_dns_nosuchname(struct lunartarget_s *query,
			      int udp_port, int query_id, char family);
NS_EXTERN char *dotted_domain;
#define domain_name  (dotted_domain + 1)
#endif

#ifdef LUNAR_HAS_GW
NS_EXTERN void lunar_do_dp(sapfpkt *pkt, void *aux);	// dp: dissemination protocol
NS_EXTERN void lunar_send_gwinfo(struct in6_addr *globaladdr);
NS_EXTERN void lunar_dsdn_conflict(sapfpkt *pkt, void *aux);
NS_EXTERN void lunar_dsdn_resolved(sapfpkt *pkt, void *aux);
NS_EXTERN void lunar_dsdn_setbidir(sapfpkt *pkt, void *aux);
NS_EXTERN void lunar_dsdn_findname(char *peername);
NS_EXTERN void lunar_dsdn_reduce(void);
NS_EXTERN int gw_flag;		// set to 1 when node is a gateway
NS_EXTERN netbox_timer_t gwtimer;
#endif

#define MAX_DSDN 6
NS_EXTERN uint32_t lunar_get_int(uint32_t min, uint32_t max);
NS_EXTERN int lunar_parse_dsdnparams(char *params, char **array);
NS_EXTERN int lunar_build_name(void);
NS_EXTERN char dsdname[128];
NS_EXTERN char dsdnparamscopy[128];
NS_EXTERN void lunar_send_name(void);
NS_EXTERN netbox_timer_t nametimer;
NS_EXTERN char *dsdnarray[MAX_DSDN];
NS_EXTERN netbox_timer_t dsdntimer;

NS_EXTERN int conf_min_ring;
NS_EXTERN int conf_max_ring;
NS_EXTERN int conf_timeout_per_ring;
NS_EXTERN int conf_max_retries;
NS_EXTERN int conf_retry_wait;
NS_EXTERN int conf_fri;
NS_EXTERN int conf_fri_jitter;
NS_EXTERN int conf_rreq_gap;
NS_EXTERN int conf_data_gap;
NS_EXTERN char bc_buf[2000];
NS_EXTERN int bc_len;
NS_EXTERN sapfsel_t bc_sel;
NS_EXTERN sapfsel_t *bc_selp;
NS_EXTERN int mysapfhandle;
NS_EXTERN struct net_device *lunardev;
NS_EXTERN struct net_device *basedev;
NS_EXTERN sapfsel_t ipv4deliver;
NS_EXTERN sapfsel_t bcast_selector;

NS_EXTERN int debuglevel;
NS_EXTERN char *basedevname;
NS_EXTERN char *forwarding;
NS_EXTERN int dgwmode;
NS_EXTERN char *dsdnparams;
NS_EXTERN int dsdnvector;

//END OF LUNAR_DECLARATIONS


#endif // LUNAR_DECL_H

// eof
