/* platform.h
 *
 *   LUNAR - Lightweight Underlay Network Ad hoc Routing
 *
 *   header file with platform specific definitions
 *
 * Copyright (C) 2001-2005 <christian.tschudin@unibas.ch>
 *               2004 <evgeny.osipov@unibas.ch>
 *
 * This file is distributed under the terms of the GNU general Public
 * License (GPL), see the file COPYING
 */


#ifndef PLATFORM_H
#define PLATFORM_H

// macros for controlling inclusion of IPV6, DHCP and DNS package.
// Defaults:

#define LUNAR_HAS_DHCP
#define LUNAR_HAS_IPV6
#define LUNAR_HAS_DNS

// added by Christophe Jelger: gatewaying support
#define LUNAR_HAS_GW

// opt out switches: (these can be set via compile options)

// #define NO_DEBUG

#ifdef NO_DHCP
# undef LUNAR_HAS_DHCP
#endif

#ifdef NO_IPV6
# undef LUNAR_HAS_IPV6
#endif

#ifdef NO_DNS
# undef LUNAR_HAS_DNS
#endif

#ifdef NO_GW
# undef LUNAR_HAS_GW
#endif

// -----------------------------------------------------------------

#ifdef NS_PORT
# undef LUNAR_HAS_DHCP
# undef LUNAR_HAS_IPv6
# undef LUNAR_HAS_DNS
# undef LUNAR_HAS_GW
# include "platform_ns2.h"
#else

//General settings for both LNX/WND
#define NS_CLASS
#define NS_CLASS_addr
#define NS_CLASS_Instance_PTR
#define SAPF_CLASS
#define SAPF_Instance
#define NETBOX_CLASS
#define NETBOX_Instance
#define _VOID_VOID_ (void(*)(void*)) 
#define PACKET_TYPE 
#define NS_STATIC static
#define NS_EXTERN extern

// other common LNX/WIN stuff
#define htonsel(s) cpu_to_be64(s)
#define ntohsel(s) be64_to_cpu(s)

#ifdef WIN32
#  include "platform_win.h"
#else // Linux (Intel and ARM)
#  include "platform_lnx.h"
#endif

#ifndef MAX
#  define MAX(a,b) ((a) < (b) ? (b) : (a))
#  define MIN(a,b) ((a) < (b) ? (a) : (b))
#endif

#endif // !NS_PORT

// debug levels:
enum {
	DBG_NONE,
	DBG_WARN,
	DBG_L2,
	DBG_L3,
	DBG_LUNAR,	// 4 RREQ+RREP send/receive
	DBG_SERVICE,	// 5 packet exchange at L2.5
	DBG_LINK,	// 6 packet exchange at L2
	DBG_L7,
	DBG_PROC,	// 8 procedure calls
	DBG_INFO,	// 9 processing info
	DBG_ALL
};

#endif // PLATFORM_H
