/* platform_lnx.h
 *
 *   LUNAR - Lightweight Underlay Network Ad hoc Routing
 *
 *   header file with LINUX specific definitions
 *
 * Copyright (C) 2001-2004 <christian.tschudin@unibas.ch>
 *
 * This file is distributed under the terms of the GNU general Public
 * License (GPL), see the file COPYING
 */

#ifndef PLATFORM_LNX_H
#define PLATFORM_LNX_H

#ifndef __KERNEL__
#  define __KERNEL__
#endif

#ifndef __OPTIMIZE__
#  define __OPTIMIZE__
#endif

#ifndef MODULE
#  define MODULE
#endif

#ifdef CONFIG_MODVERSIONS
#  define MODVERSIONS
//#  include <linux/modversions.h>
#endif

//#include <linux/config.h>
#include <linux/module.h>
// ----------------------------------------------------------------------

#ifndef MODULE_LICENSE
# define MODULE_LICENSE(s)  typedef char dummy782378273
#endif

#include <linux/kernel.h>
#include <linux/init.h>

#ifdef CONFIG_MIPS
# undef CONFIG_CPU_HAS_LLSC
#endif

#include <asm/atomic.h>
#include <asm/byteorder.h>
#include <asm/bitops.h>
#include <linux/version.h>
#include <linux/ctype.h>
#include <linux/utsname.h>

#if defined(CONFIG_ARM)
// || defined(CONFIG_MIPS)
# define free_netdev(a)		do{}while(0)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,6)
# define UP_MODULE_USE()	MOD_INC_USE_COUNT
# define DOWN_MODULE_USE()	MOD_DEC_USE_COUNT
#else
# define UP_MODULE_USE()	try_module_get(THIS_MODULE)
# define DOWN_MODULE_USE()	module_put(THIS_MODULE)
#endif

// ----------------------------------------------------------------------

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,10)

void __bug(const char *file, int line, void *data){
  printk("bug: %s %d\n", file, line);
}

#define _set_bit_le    ___set_bit_le
#define _clear_bit_le  ___clear_bit_le

__inline__ __const__ __u16 __fswab16(__u16 x) {
	return (x << 8) | ((x >> 8) & 0x0ff);
}

__inline__ __const__ __u32 __fswab32(__u32 x) {
	return
		(((__u32)x & (__u32)0x000000ffUL) << 24) |
		(((__u32)x & (__u32)0x0000ff00UL) <<  8) |
		(((__u32)x & (__u32)0x00ff0000UL) >>  8) |
		(((__u32)x & (__u32)0xff000000UL) >> 24) ;
}

__inline__ __const__ __u64 __fswab64(__u64 x) {
	return
                (((__u64)x & (__u64)0x00000000000000ffULL) << 56) |
                (((__u64)x & (__u64)0x000000000000ff00ULL) << 40) |
                (((__u64)x & (__u64)0x0000000000ff0000ULL) << 24) |
                (((__u64)x & (__u64)0x00000000ff000000ULL) <<  8) |
                (((__u64)x & (__u64)0x000000ff00000000ULL) >>  8) |
                (((__u64)x & (__u64)0x0000ff0000000000ULL) >> 24) |
                (((__u64)x & (__u64)0x00ff000000000000ULL) >> 40) |
                (((__u64)x & (__u64)0xff00000000000000ULL) >> 56) ;
}

#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
#  define INIT_ETHERDEV(dev,len) init_etherdev((dev),(len))
#  define REGISTER_NETDEV(dev) NULL
#else
// #  define INIT_ETHERDEV(dev,len) alloc_etherdev(len)
#  define INIT_ETHERDEV(dev,len) alloc_netdev(len, "lunar%d", ether_setup)
#  define REGISTER_NETDEV(dev) register_netdev(dev)
#endif

// #if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,8)
// #  define strsep(s,delim) strtok((s), (delim))
// #endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
#  define AF_PACKET_PRIV(p) ((p)->data)
#else
#  define AF_PACKET_PRIV(p) ((p)->af_packet_priv)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,9)
#  define ETHSRC(skb)               ((skb)->mac.ethernet->h_source)
#  define NEIGH_UPDATE(n,p,a,b,c)   neigh_update(n,p,a,b,c)
#else
#  define ETHSRC(skb)               (eth_hdr(skb)->h_source)
#  define NEIGH_UPDATE(n,p,a,b,c)   neigh_update(n,p,a,b|c)
#endif

// ----------------------------------------------------------------------

#include <linux/errno.h>
#include <linux/types.h>
#include <linux/socket.h>
#include <linux/proc_fs.h>
#include <linux/random.h>
#include <linux/timer.h>

#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/inetdevice.h>
#include <linux/in.h>

#include <linux/ip.h>
#include <linux/in6.h> // needed for include/asm/chksum.h
#include <linux/udp.h>

#ifdef LUNAR_HAS_IPV6
#  include <linux/icmpv6.h>
#  include <net/ipv6.h>
#  include <net/if_inet6.h>
#  include <net/ndisc.h>

// scope fc00::/7
#define IPV6_ADDR_SCOPE_MANET 0x04

// ------ Copied from /usr/include/netinet/in.h

#define IN6_IS_ADDR_UNSPECIFIED(a) \
	(((__const uint32_t *) (a))[0] == 0				      \
	 && ((__const uint32_t *) (a))[1] == 0				      \
	 && ((__const uint32_t *) (a))[2] == 0				      \
	 && ((__const uint32_t *) (a))[3] == 0)

#define IN6_IS_ADDR_MULTICAST(a) (((__const uint8_t *) (a))[0] == 0xff)

#define IN6_IS_ADDR_LINKLOCAL(a) \
	((((__const uint32_t *) (a))[0] & htonl (0xffc00000))		\
	 == htonl (0xfe800000))

#define IN6_IS_ADDR_SITELOCAL(a) \
	((((__const uint32_t *) (a))[0] & htonl (0xffc00000))		\
	 == htonl (0xfec00000))

#define IN6_IS_ADDR_MANETLOCAL(a) \
	((((__const uint32_t *) (a))[0] & htonl (0xfc000000))		\
	 == htonl (0xfc000000))

#endif // LUNAR_HAS_IPV6

#include <asm/uaccess.h>
#include <asm/checksum.h>

// ----------------------------------------------------------------------

inline unsigned char getRandomByte(void)
{
	unsigned char b;

	get_random_bytes(&b, 1);
	return b;
}

#define arm_timer(tp,ams) do{(tp)->expires=(ams);add_timer(tp);}while(0)
#define cancel_timer(tp)  del_timer(tp)

# define const64(x) (x##LL)
# define const64u(x) (x##ULL)

#ifndef NO_DEBUG
# define SAPFDEBUG(l,s)		do {if ((l) <= sapfdebug) {s;} } while (0)
# define DEBUG(l,s)		do {if ((l) <= debuglevel) {s;} } while (0)
# define DEBUG_TARGET(l,s,t)	do {if ((l) <= debuglevel) \
					debug_target(s,t);\
				} while (0)
  static int debuglevel = 0;
#else
# define SAPFDEBUG(l,s)		do {} while (0)
# define DEBUG(l,s)		do {} while (0)
# define DEBUG_TARGET(l,s,t)	do {} while (0)
#endif

#endif // PLATFORM_LNX_H
