/* platform_win.h
 *
 *   LUNAR - Lightweight Underlay Network Ad hoc Routing
 *
 *   header file with platform specific definitions
 *
 * Copyright (C) 2003      <oskarw@it.uu.se>
 * Copyright (C) 2004      <christian.tschudin@unibas.ch>
 *
 *  2004-05-09 cft: cleansed from old emulation balast
 *  2004-03-18 cft: extracted from platform-h
 *  2003-11-18 Oskar: added Windows stuff
 *
 * This file is distributed under the terms of the GNU general Public
 * License (GPL), see the file COPYING
 */

#ifndef PLATFORM_WIN_H
#define PLATFORM_WIN_H

#include <stdlib.h>
#include <stdio.h>
#include <memory.h>

#pragma pack(1)

typedef unsigned char __u8;
typedef unsigned char BYTE;
typedef __u8 u_int8_t;
typedef __u8 uint8_t;
typedef unsigned short __u16;
typedef __u16 u_int16_t;
typedef __u16 uint16_t;
typedef unsigned int u32;
typedef unsigned int __u32;
typedef __u32 u_int32_t;
typedef __u32 uint32_t;
typedef int int32_t;
typedef unsigned __int64 __u64;
typedef __u64 uint64_t;
typedef long off_t;

/*
typedef struct {
	volatile int counter;
} atomic_t;
typedef unsigned short __kernel_mode_t;
typedef __kernel_mode_t mode_t;
typedef int (get_info_t) (char *, char **, off_t, int);
*/

#define const64(x) (x##i64)
#define const64u(x) (x##Ui64)

__inline void set_bit(int nr, volatile void *addr)
{
	((BYTE *) addr)[nr / 8] |= 1 << (nr % 8);
}

__inline int test_bit(int nr, volatile void *addr)
{
	return ( (((BYTE *) addr)[nr / 8] & (1 << (nr % 8))) != 0 );
}

__inline const __u16 __fswab16(__u16 x)
{
	return (x << 8) | ((x >> 8) & 0x0ff);
}

__inline const __u32 __fswab32(__u32 x)
{
	return
	    (((__u32) x & (__u32) 0x000000ffUL) << 24) |
	    (((__u32) x & (__u32) 0x0000ff00UL) << 8) |
	    (((__u32) x & (__u32) 0x00ff0000UL) >> 8) |
	    (((__u32) x & (__u32) 0xff000000UL) >> 24);
}

__inline const __u64 __fswab64(__u64 x)
{
	return
	    (((__u64) x & (__u64) 0x00000000000000ffUi64) << 56) |
	    (((__u64) x & (__u64) 0x000000000000ff00Ui64) << 40) |
	    (((__u64) x & (__u64) 0x0000000000ff0000Ui64) << 24) |
	    (((__u64) x & (__u64) 0x00000000ff000000Ui64) << 8) |
	    (((__u64) x & (__u64) 0x000000ff00000000Ui64) >> 8) |
	    (((__u64) x & (__u64) 0x0000ff0000000000Ui64) >> 24) |
	    (((__u64) x & (__u64) 0x00ff000000000000Ui64) >> 40) |
	    (((__u64) x & (__u64) 0xff00000000000000Ui64) >> 56);
}

#define __swab16(x) __fswab16(x)
#define __swab32(x) __fswab32(x)
#define __swab64(x) __fswab64(x)

#define __cpu_to_be16(x) __swab16((x))
#define __be16_to_cpu(x) __swab16((x))
#define __cpu_to_be32(x) __swab32((x))
#define __be32_to_cpu(x) __swab32((x))
#define __cpu_to_be64(x) __swab64((x))
#define __be64_to_cpu(x) __swab64((x))

#define cpu_to_be64 __cpu_to_be64
#define be64_to_cpu __be64_to_cpu

#define __htons(x) __cpu_to_be16(x)
#define __ntohs(x) __be16_to_cpu(x)
#define htons(x) __htons(x)
#define ntohs(x) __ntohs(x)
#define ntohl(x) __be32_to_cpu(x)
#define htonl(x) __cpu_to_be32(x)

#define UP_MODULE_USE()   do{}while(0)
#define DOWN_MODULE_USE() do{}while(0)

#define HZ 1000
#define jiffies get_millis()	/* unsigned long volatile */
#define time_after(a,b) ((long)(b) - (long)(a) < 0)

struct timer_list {
	unsigned long data;
	void (*function) (unsigned long);
	NDIS_MINIPORT_TIMER mptimer;
};


#define DEBUG(l,s) {if((l)<=debuglevel){s;}}
extern int debuglevel;
#define printk DbgPrint

/* Ether types */
#define ETH_P_ARP 0x0806  /* Address Resolution packet */
#define ETH_P_IP  0x0800  /* Internet Protocol packet */  

/* 10Mb/s ethernet header */
#define ETH_ALEN 6       
#define ETH_HLEN 14       /* Total octets (bytes) in header */

struct ether_header {
	u_int8_t ether_dhost[ETH_ALEN];	/* destination eth addr */
	u_int8_t ether_shost[ETH_ALEN];	/* source ether addr    */
	u_int16_t ether_type;	/* packet type ID field */
};

/* ARP protocol opcodes */
#define	ARPOP_REQUEST  0x0001	/* ARP request  */
#define	ARPOP_REPLY    0x0002	/* ARP reply    */
#define	ARPOP_RREQUEST 0x0003	/* RARP request */
#define	ARPOP_RREPLY   0x0004	/* RARP reply   */

/* Ethernet arp pkt */
struct arp_pkt {
	unsigned short ar_hrd;	/* format of hardware address */
	unsigned short ar_pro;	/* format of protocol address */
	unsigned char ar_hln;	/* length of hardware address */
	unsigned char ar_pln;	/* length of protocol address */
	unsigned short ar_op;	/* ARP opcode (command)       */
	unsigned char ar_sha[ETH_ALEN];	/* sender hardware address    */
	unsigned char ar_sip[4];	/* sender IP address          */
	unsigned char ar_tha[ETH_ALEN];	/* target hardware address    */
	unsigned char ar_tip[4];	/* target IP address          */
};

struct iphdr {
	/* Note: Little endian */
	__u8 ihl:4, version:4;
	__u8 tos;
	__u16 tot_len;
	__u16 id;
	__u16 frag_off;
	__u8 ttl;
	__u8 protocol;
	__u16 check;
	__u32 saddr;
	__u32 daddr;
	/* The options start here */
};

struct udphdr {
	__u16 source;
	__u16 dest;
	__u16 len;
	__u16 check;
};

struct in_addr {
	__u32 s_addr;
};

/* Supported address families */
#define AF_UNSPEC               0
struct sockaddr {
	unsigned short sa_family;	/* address family, AF_xxx */
	char sa_data[14];	/* 14 bytes of protocol address */
};

/* This struct is very slimmed and just used to fit
   the Windoze port */
struct net_device_stats {
	unsigned long rx_packets;	/* total packets received */
	unsigned long tx_packets;	/* total packets transmitted */
	unsigned long rx_bytes;
	unsigned long tx_bytes;
};

/* This struct is very slimmed and just used to fit
   the Windoze port */
struct net_device {
	/* Interface index. Unique device identifier */
	int ifindex;

	/* pointer to private data */
	void *priv;
};


/* Some NDIS stuff */
/* Structure used for modeling queued (ethernet)
   packets */
typedef struct _QUEUED_PACKET {
	PNDIS_PACKET packet;
	LIST_ENTRY listEntry;
} QUEUED_PACKET, *PQUEUED_PACKET;

/* Protocol reserved part of the packet */
typedef struct _PROTOCOL_RESERVED {
	PNDIS_PACKET OriginalPkt;
} RSVD, *PRSVD;

#endif				// PLATFORM_WIN_H
