/* lunar_gw.c
 *
 *   LUNAR - Lightweight Underlay Network Ad hoc Routing
 *
 *   Gatewaying and global address configuration
 *
 * Copyright (C) 2005 <christophe.jelger@unibas.ch>
 *
 *   2005-0-26 cj: first version
 *
 * This file is distributed under the terms of the GNU general Public
 * License (GPL), see the file COPYING
 */

#ifdef LUNAR_HAS_GW
#define GWINFO_PERIOD 30000000
#define GWINFO_VALID  40000000
#define RA_VALID 40
#define MIN_NAME_JITTER 200000
#define MAX_NAME_JITTER 500000

struct ipv6_gw {
	struct in6_addr gw_addr;
	struct in6_addr rt_addr;
} my_ipv6_gw;

uint32_t NS_CLASS lunar_get_int(uint32_t min, uint32_t max)
{
	uint32_t rnd;
	
	get_random_bytes(&rnd, 4);

	if (max)
		return (uint32_t)( min + rnd%(max-min) );
	else
		return (uint32_t)( rnd > min ? rnd : (rnd+min) );
}

void NS_CLASS lunar_gw_expires(struct ipv6_gw *gw)
{
	DEBUG(DBG_LUNAR, printk("SAA: gwinfo expired !"));
	
	lunar_ipv6_send_RA(&gw->gw_addr, &gw->rt_addr, 0, 0);
}

void NS_CLASS lunar_do_dp(sapfpkt *p, void *aux)
{
	struct xrp_head_s *x = (struct xrp_head_s *) p->data;
	xrp_param_t *pp; // pointer to a single parameter block
	int cnt, len, ndx = -1;
	struct lunartarget_s gwaddr, rtaddr, myname, myhit;
	struct lunartarget_s s_name, *srcname=0, s_hit, *srchit=0;
	struct xrp_sel_s *series = 0;
	int ifnr = basedev ? basedev->ifindex : 0;
	
	/* for concast tree */
	struct xrp_sel_eth_s *reply = 0;
	sapfsel_t backsel;
	struct sockaddr sa;
	struct uc_transit_pending_s *tr;
	
	memset(&gwaddr, 0, sizeof(struct lunartarget_s));
	memset(&rtaddr, 0, sizeof(struct lunartarget_s));
	
	DEBUG(DBG_LUNAR, printk("DP: received data (%d bytes)\n",  p->len));
	
	if (p->len < sizeof(struct xrp_head_s) ||
	    ntohs(x->cmd) != XRP_CMD_LUNAR_DP) {
		DEBUG(DBG_INFO, printk(" wrong XRP cmd (not DP)\n"));
		return;
	}
	// parse GW:
	len = p->len - sizeof(*x);
	pp = (xrp_param_t *) (x+1);
	while (len >= 2) {
		if (ntohs(pp->param.len) & XRP_CMD_BIT)
			break;
		if (len < 4 ||
		    ntohs(pp->param.len) > len ||
		    ntohs(pp->param.len) < 4) {
			DEBUG(DBG_INFO, printk(" end of XRP parsing: wrong length %d\n", len));
			return;
		}
		if (pp->param.partype > XRP_CTYPE_UNKNOWN &&
		    pp->param.partype < XRP_CTYPE__COUNT) {
			if (pp->param.partype != XRP_CTYPE_NAME &&
			    ntohs(pp->param.len) !=
			    		xrp_param_len[pp->param.partype]) {
				DEBUG(DBG_INFO, printk(" wrong parameter length. class=%d, type=%d, len=%d, explen=%d\n", pp->param.parclass, pp->param.partype, ntohs(pp->param.len), pp->param.partype));
				return;
			}
			DEBUG(DBG_ALL, printk(" param class+len+expect_len: %d %d %d\n",
			pp->param.parclass, ntohs(pp->param.len), xrp_param_len[pp->param.partype]));

			switch (pp->param.parclass) {
			case XRP_CLASS_LUNAR_GW:
				switch(pp->param.partype) {
				case XRP_CTYPE_SELECTOR:
					series = (struct xrp_sel_s *) pp;
					break;
				case XRP_CTYPE_IPv4:
				case XRP_CTYPE_IPv6:
					if (!param2target(&gwaddr, pp)) {
						DEBUG_TARGET(10, "GWINFO: got IP6", &gwaddr);
						
						// first addr received is link-local
						
						if (IN6_IS_ADDR_LINKLOCAL(&gwaddr.id.ipv6))
							memcpy( &rtaddr, &gwaddr, sizeof(struct lunartarget_s));
					}
					break;
				case XRP_CTYPE_NAME:
					if (!param2target(&s_name, pp)) {
						srcname = &s_name;
						DEBUG_TARGET(10, "GWINFO: got name", srcname);
					}
					break;
				case XRP_CTYPE_HIT128:
					if (!param2target(&s_hit, pp)) {
						srchit = &s_hit;
						DEBUG_TARGET(10, "GWINFO: got hit", srchit);
					}
					break;
				default:
					break;
				}
				break;
			case XRP_CLASS_LUNAR_HELLO:
				switch(pp->param.partype) {
				case XRP_CTYPE_SELECTOR:
					series = (struct xrp_sel_s *) pp;
					break;
				case XRP_CTYPE_NAME:
					if (!param2target(&s_name, pp)) {
						srcname = &s_name;
						DEBUG_TARGET(10, "HELLO: got name", srcname);
					}
					break;
				case XRP_CTYPE_HIT128:
					if (!param2target(&s_hit, pp)) {
						srchit = &s_hit;
						DEBUG_TARGET(10, "HELLO: got hit", srchit);
					}
					break;
				default:
					break;
				}
				break;
			case XRP_CLASS_LUNAR_DSDN_REPLYTO:
				if (pp->param.partype == XRP_CTYPE_SEL_ETH)
					reply = (struct xrp_sel_eth_s *) pp;
				break;
			default: // ignore unknown fields
				break;
			}
		}
		cnt = (ntohs(pp->param.len) + 3) & ~0x3;
		len -= cnt;
		pp = (xrp_param_t*) (((char*)pp) + cnt);
	}
	
	// prevent broadcast storms:
	if (!series || SAPF_Instance sapf_exists(mysapfhandle, series->sel)) {
		DEBUG(DBG_INFO, printk(series ? "  series %016Lx already exists, dropping\n" : "  no series\n", series ? ntohsel(series->sel) : 0L));
		return;
	}
	SAPF_Instance sapf_addREDIR(mysapfhandle, series->sel, 0);
	
	netbox_getL3addr(&myname, lunardev, AF_NAME);
	netbox_getL3addr(&myhit, lunardev, AF_HIT);
	/* add sender of HELLO msg */
	if (srcname && !TARGET_EQUAL(srcname, &myname) && !TARGET_EQUAL(srchit, &myhit)) {
		
		ndx = peer_install(srchit, 0, srcname);
		
		if (ndx > -1)
		{
			NETBOX_Instance netbox_deltimer(&(peers[ndx].timer));
			NETBOX_Instance netbox_newtimer(&(peers[ndx].timer),
					_VOID_VOID_ peer_timeout,
					&peers[ndx], LUNAR_DNS_TIMEOUT);
				
			if (!IN6_IS_ADDR_UNSPECIFIED(&gwaddr.id.ipv6))
				peers[ndx].flags |= PEER_IS_GW;
			else
				peers[ndx].flags &= ~PEER_IS_GW;
		}
	}
	else if (srcname && TARGET_EQUAL(srcname, &myname))
	{
		/* somebody is using the same name as me */
		char buf[300], *cp = buf;
		struct sockaddr lsa;
		sapfsel_t myreply;
		struct sockaddr sa;		
		
		DEBUG(DBG_LUNAR, printk("DSDN: name conflict detected!\n"));
		myreply = mkHandlerPointer(NS_CLASS_addr lunar_dsdn_resolved, 0);
		
		if (reply)
		{
			cp += xrp_fill_hdr(cp, XRP_CMD_LUNAR_DSDN, 0);
			NETBOX_Instance netbox_getL2addr(ifnr, &sa);
			cp += xrp_fill_sel_eth(cp, XRP_CLASS_LUNAR_DSDN_REPLYTO,
			       myreply, (unsigned char *)sa.sa_data);
			cp += xrp_fill_name(cp, XRP_CLASS_LUNAR_DSDN_CONFLICT, dsdnparams);
			cp += xrp_fill_hit128(cp, XRP_CLASS_LUNAR_DSDN_CONFLICT, &myhit.id.hit);
			cp += xrp_fill_eom(cp);
		
			memcpy( lsa.sa_data, reply->eth, ETH_ALEN );
			NETBOX_Instance netbox_l2send(ifnr, &lsa, reply->sel, buf, cp - buf);
		}
		else
			DEBUG(DBG_LUNAR, printk("DSDN: no reply eth+sel given\n"));
	}
					
	/* add IPv6 gateway */
	if( ndx > -1 && !IN6_IS_ADDR_UNSPECIFIED(&gwaddr.id.ipv6) && !gw_flag && IamIPv6Ready() )
	{
		unsigned long t = GWINFO_VALID;	/* 40 seconds */
		
		lunar_ipv6_send_RA(&gwaddr.id.ipv6, &rtaddr.id.ipv6, RA_VALID, 0xff);
		
		my_ipv6_gw.gw_addr = gwaddr.id.ipv6;
		my_ipv6_gw.rt_addr = rtaddr.id.ipv6;
		
		/* addr and default route are suppressed if not periodically refreshed */
				
		NETBOX_Instance netbox_deltimer(&gwtimer);
		NETBOX_Instance netbox_newtimer(&gwtimer, 
				_VOID_VOID_ lunar_gw_expires, &my_ipv6_gw, t);
	}
		
	/* Setup concast entry towards root and resend packet */
		
	tr = (struct uc_transit_pending_s *)kmalloc(sizeof(*tr), GFP_ATOMIC);
	if (!tr)
		return;
	memset(tr, 0, sizeof(*tr));
	
	tr->replysel = reply->sel;
	memcpy(tr->replyeth, reply->eth, ETH_ALEN);
	tr->caller = backsel = mkHandlerPointer(NS_CLASS_addr lunar_dsdn_setbidir, tr);
		
	//backsel = mkNetPointer(reply->eth, reply->sel);
	NETBOX_Instance netbox_getL2addr(ifnr, &sa);
	memcpy( reply->eth, sa.sa_data, ETH_ALEN );
	reply->sel = backsel;
	
	NETBOX_Instance netbox_l2send(ifnr, 0,
				      htonsel(SAPF_SELNET_LUNAR_BCAST),
				      p->data, p->len);
}

void NS_CLASS lunar_send_gwinfo(struct in6_addr *globaladdr)
{
	int ifnr = basedev ? basedev->ifindex : 0;
	char buf[300], *cp = buf;
	struct in6_addr ipv6_local = IN6ADDR_FAKE_SRC;	// has a local scope
	struct lunartarget_s my_ipv6_local, myname, myhit;
	sapfsel_t series;
	unsigned long t = GWINFO_PERIOD;	/* 30 seconds */

	/* first time the globaladdr is given */

	if( !globaladdr ) {
	
		struct inet6_dev *in6dev = (struct inet6_dev*) lunardev->ip6_ptr;
		struct inet6_ifaddr *cur;
		
		for (cur = in6dev->addr_list; cur; cur = cur->if_next) {
		
			if( cur->scope == IFA_GLOBAL && cur->flags&IFA_F_PERMANENT && 
			    gw_flag && !IN6_IS_ADDR_MANETLOCAL(&cur->addr) ) {
			    	
				globaladdr = &cur->addr;
				break;
			}
		}
	}

	if( !globaladdr ) {
	
		DEBUG(DBG_LUNAR, printk("GW: failed to find global addr\n"));
		gw_flag = 0;	/* not a gateway any more */
		lunar_send_name();
		return;
	}
	
	gw_flag = 1;

	DEBUG(DBG_LUNAR, printk("GW: sending gwinfo\n"));
	
	// we need our link-local address
	memcpy( &my_ipv6_local.id.ipv6, &ipv6_local, sizeof(struct in6_addr));
	my_ipv6_local.family = AF_INET6;
	netbox_getL3addr(&my_ipv6_local, lunardev, AF_INET6);
	
	// we need our name and hit
	netbox_getL3addr(&myname, lunardev, AF_NAME);
	netbox_getL3addr(&myhit, lunardev, AF_HIT);
	
	// to avoid broadcast storm
	
	series = SAPF_Instance sapf_getUnusedSelector(mysapfhandle, SELCLASS_REMOTE);
	SAPF_Instance sapf_addREDIR(mysapfhandle, series, 0);
	
	cp += xrp_fill_hdr(cp, XRP_CMD_LUNAR_DP, 0);
	cp += xrp_fill_sel(cp, XRP_CLASS_LUNAR_GW, series);
	cp += xrp_fill_ipv6(cp, XRP_CLASS_LUNAR_GW, &my_ipv6_local.id.ipv6);
	cp += xrp_fill_ipv6(cp, XRP_CLASS_LUNAR_GW, globaladdr);
	cp += xrp_fill_name(cp, XRP_CLASS_LUNAR_GW, myname.id.name);
	cp += xrp_fill_hit128(cp, XRP_CLASS_LUNAR_GW, &myhit.id.hit);
	cp += xrp_fill_eom(cp);

	NETBOX_Instance netbox_l2send(ifnr, 0,
				      htonsel(SAPF_SELNET_LUNAR_BCAST),
				      buf, cp - buf);
				      
	NETBOX_Instance netbox_deltimer(&gwtimer);
	NETBOX_Instance netbox_newtimer(&gwtimer, _VOID_VOID_ lunar_send_gwinfo, 0, t);
	
}

#endif // LUNAR_HAS_GW

void NS_CLASS lunar_send_name(void)
{
	int ifnr = basedev ? basedev->ifindex : 0;
	char buf[300], *cp = buf;
	struct lunartarget_s myname, myhit;
	sapfsel_t series, reply;
	struct sockaddr sa;
	int rnd;
	
	if (gw_flag)
		return;	// gateway's name+hit is sent with GWINFO
	
	DEBUG(DBG_LUNAR, printk("HELLO: broadcasting name info\n"));
	
	// we need our name and hit
	netbox_getL3addr(&myname, lunardev, AF_NAME);
	netbox_getL3addr(&myhit, lunardev, AF_HIT);
	
	// to avoid broadcast storm
	
	series = SAPF_Instance sapf_getUnusedSelector(mysapfhandle, SELCLASS_REMOTE);
	SAPF_Instance sapf_addREDIR(mysapfhandle, series, 0);
	
	// for potential conflict message sent back 
	
	reply = mkHandlerPointer(NS_CLASS_addr lunar_dsdn_conflict, 0);
	
	cp += xrp_fill_hdr(cp, XRP_CMD_LUNAR_DP, 0);
	cp += xrp_fill_sel(cp, XRP_CLASS_LUNAR_HELLO, series);
	NETBOX_Instance netbox_getL2addr(ifnr, &sa);
	cp += xrp_fill_sel_eth(cp, XRP_CLASS_LUNAR_DSDN_REPLYTO,
			       reply, (unsigned char *)sa.sa_data);
	cp += xrp_fill_name(cp, XRP_CLASS_LUNAR_HELLO, myname.id.name);
	cp += xrp_fill_hit128(cp, XRP_CLASS_LUNAR_HELLO, &myhit.id.hit);
	cp += xrp_fill_eom(cp);

	NETBOX_Instance netbox_l2send(ifnr, 0,
				      htonsel(SAPF_SELNET_LUNAR_BCAST),
				      buf, cp - buf);
	
	rnd = LUNAR_NAME_TIMEOUT + lunar_get_int(MIN_NAME_JITTER, MAX_NAME_JITTER);
	      
	NETBOX_Instance netbox_deltimer(&nametimer);
	NETBOX_Instance netbox_newtimer(&nametimer, 
			_VOID_VOID_ lunar_send_name, 0, rnd);
	
}

void NS_CLASS lunar_dsdn_reduce()
{
	int last, i, cur, rnd;
	char peername[LUNAR_MAX_HOSTNAME_LEN + 1];
	char mytmpname[LUNAR_MAX_HOSTNAME_LEN + 1];
	
	/* re-schedule function call */
	
	NETBOX_Instance netbox_deltimer(&dsdntimer);
	NETBOX_Instance netbox_newtimer(&dsdntimer, 
			_VOID_VOID_ lunar_dsdn_reduce, 0, LUNAR_DSDN_TIMEOUT);
	
	/* periodically try to shrink name */
	DEBUG(DBG_LUNAR, printk("lunar_dsdn_reduce: vector = %d\n", dsdnvector));
	
	if (dsdnvector <= 12)
		return;		/* name cannot be shortened */
		
	/* find last element added to name */	

	last = dsdnvector%10-1;
	
	if (last >= MAX_DSDN)
		return;
		
	strcpy(mytmpname, dsdname);
	mytmpname[strlen(mytmpname)-strlen(dsdnarray[last])-1] = '\0'; // remove "-last" parameter
	
	for (i = 0; i < LUNAR_MAX_NAMES; i++) {
		
		if (namedb[i].name.family != AF_NAME)
			continue;
		
		strcpy(peername, namedb[i].name.id.name);
		cur = strlen(peername);		
			
		/* remove "-last" parameter */
		
		while (cur--)
		{
			if (peername[cur] == '-')
			{
				peername[cur] = '\0';
				break;
			}
		}
		
		DEBUG(DBG_LUNAR, printk("lunar_dsdn_reduce: checking %s\n", peername));
		
		if (!strcmp(mytmpname, peername))
		{
			DEBUG(DBG_LUNAR, printk("lunar_dsdn_reduce: cannot reduce name\n"));
			return;		/* cannot remove last parameter */
		}
	}
	
	/* last dsdn parameter can be removed */

	strcpy(dsdname, mytmpname);
	DEBUG(DBG_LUNAR, printk("lunar_dsdn_reduce: name reduced to %s\n", dsdname));
	dsdnvector = 12;
	
	rnd = lunar_get_int(MIN_NAME_JITTER, MAX_NAME_JITTER);
	      
	NETBOX_Instance netbox_deltimer(&nametimer);
	NETBOX_Instance netbox_newtimer(&nametimer, 
			_VOID_VOID_ lunar_send_name, 0, rnd);
}

void NS_CLASS lunar_dsdn_findname(char *peername)
{
	char *params[MAX_DSDN];
	int cnt, mine, his;
	char tmpname[LUNAR_MAX_HOSTNAME_LEN + 1];
	
	cnt = lunar_parse_dsdnparams(peername, params); 
		
	if (!cnt)
	{
		DEBUG(DBG_LUNAR, printk("lunar_dsdn_findname: attribute list is empty!\n"));
		return;
	}
	
	/* by default the first 2 attributes are always kept */
		
	for (mine=2; mine<MAX_DSDN; mine++)
	{
		for (his=2; his<cnt; his++)
		{
			if (!strcmp(dsdnarray[mine], params[his]) )
				break;
		}
		
		if (his == cnt)
		{
			struct lunartarget_s try;
			try.family = AF_NAME;
			
			strcpy(tmpname, dsdname);
			strcpy(tmpname+strlen(tmpname), "-");
			strcpy(tmpname+strlen(tmpname), dsdnarray[mine]);
			
			/* check that new name is not used by any other LUNAR node */
			/* append .net.lunar first */
			
			strcpy(try.id.name, tmpname);
			strcpy(try.id.name+strlen(try.id.name), ".net.lunar");
			
			if( namedb_find(&try) < 0 )
			{
				DEBUG(DBG_LUNAR, printk("lunar_dsdn_findname: name collision resolved\n"));
				strcpy(dsdname, tmpname);
				DEBUG(DBG_LUNAR, printk("--> new name is %s\n", dsdname));
				dsdnvector = dsdnvector*10 + mine + 1;
				
				/* re-schedule name reduction */
				
				NETBOX_Instance netbox_deltimer(&dsdntimer);
				NETBOX_Instance netbox_newtimer(&dsdntimer, 
						_VOID_VOID_ lunar_dsdn_reduce, 0, LUNAR_DSDN_TIMEOUT);
				
				return;
			}
			else
				DEBUG(DBG_LUNAR, printk("lunar_dsdn_findname: %s already exists\n", tmpname));
		}
	}
	
	DEBUG(DBG_LUNAR, printk("lunar_dsdn_findname: cannot find unique attribute\n"));
	DEBUG(DBG_LUNAR, printk("--> flaging node as \"name-collision\"\n"));
	
	strcpy(dsdname+strlen(dsdname), "-");
	strcpy(dsdname+strlen(dsdname), "name-collision");
	return;
}

void NS_CLASS lunar_dsdn_setbidir(sapfpkt *p, void *aux)
{
	struct xrp_head_s *x = (struct xrp_head_s *) p->data;
	xrp_param_t *pp; // pointer to a single parameter block
	int cnt, len;
	int ifnr = basedev ? basedev->ifindex : 0;
	sapfsel_t fwdsel, caller;
	struct uc_transit_pending_s *tr =(struct uc_transit_pending_s *)aux;
	
	/* to reply */
	struct xrp_sel_eth_s *reply = 0;
	struct xrp_sel_eth_s back;
	struct sockaddr sa;
	
	if (p->len < sizeof(struct xrp_head_s) ||
	    ntohs(x->cmd) != XRP_CMD_LUNAR_DSDN) {
		DEBUG(DBG_INFO, printk(" wrong XRP cmd (not DSDN)\n"));
		return;
	}
	
	len = p->len - sizeof(*x);
	pp = (xrp_param_t *) (x+1);
	while (len >= 2) {
		if (ntohs(pp->param.len) & XRP_CMD_BIT)
			break;
		if (len < 4 ||
		    ntohs(pp->param.len) > len ||
		    ntohs(pp->param.len) < 4) {
			DEBUG(DBG_INFO, printk(" end of XRP parsing: wrong length %d\n", len));
			return;
		}
		if (pp->param.partype > XRP_CTYPE_UNKNOWN &&
		    pp->param.partype < XRP_CTYPE__COUNT) {
			if (pp->param.partype != XRP_CTYPE_NAME &&
			    ntohs(pp->param.len) !=
			    		xrp_param_len[pp->param.partype]) {
				DEBUG(DBG_INFO, printk(" wrong parameter length. class=%d, type=%d, len=%d, explen=%d\n", pp->param.parclass, pp->param.partype, ntohs(pp->param.len), pp->param.partype));
				return;
			}
			DEBUG(DBG_ALL, printk(" param class+len+expect_len: %d %d %d\n",
			pp->param.parclass, ntohs(pp->param.len), xrp_param_len[pp->param.partype]));

			switch (pp->param.parclass) {
			case XRP_CLASS_LUNAR_DSDN_REPLYTO:
				if (pp->param.partype == XRP_CTYPE_SEL_ETH)
					reply = (struct xrp_sel_eth_s *) pp;
				break;
			default:
				break;
			}
		}
		cnt = (ntohs(pp->param.len) + 3) & ~0x3;
		len -= cnt;
		pp = (xrp_param_t*) (((char*)pp) + cnt);
	}
	
	fwdsel = mkNetPointer(reply->eth, reply->sel);
	NETBOX_Instance netbox_getL2addr(ifnr, &sa);
	memcpy( reply->eth, sa.sa_data, ETH_ALEN );
	reply->sel = fwdsel;
	
	if (!tr)
	{
		DEBUG(DBG_INFO, printk("lunar_dsdn_setbidir: tr NULL!\n"));
		return;
	}
	
	back.sel = tr->replysel;
	memcpy( back.eth, tr->replyeth, ETH_ALEN );
	caller = tr->caller;
		
	SAPF_Instance sapf_deleteEntry(mysapfhandle, caller);
	SAPF_Instance sapf_addCallback(mysapfhandle, caller, NS_CLASS_addr L2sendto,
			 		mkSapfDest(back.eth, back.sel));
	
	memcpy( sa.sa_data, back.eth, ETH_ALEN );
	NETBOX_Instance netbox_l2send(ifnr, &sa, back.sel, p->data, p->len);
}

void NS_CLASS lunar_dsdn_resolved(sapfpkt *p, void *aux)
{
	struct xrp_head_s *x = (struct xrp_head_s *) p->data;
	xrp_param_t *pp; // pointer to a single parameter block
	int cnt, len, rnd;
	struct lunartarget_s s_name, *srcname=0, s_hit, *srchit=0;
	
	DEBUG(DBG_LUNAR, printk("DSDN: name resolved msg!\n"));	
	DEBUG(DBG_LUNAR, printk("DSDN: received resolved msg (%d bytes)\n", p->len));
	
	if (p->len < sizeof(struct xrp_head_s) ||
	    ntohs(x->cmd) != XRP_CMD_LUNAR_DSDN) {
		DEBUG(DBG_INFO, printk(" wrong XRP cmd (not DSDN)\n"));
		return;
	}
	
	len = p->len - sizeof(*x);
	pp = (xrp_param_t *) (x+1);
	while (len >= 2) {
		if (ntohs(pp->param.len) & XRP_CMD_BIT)
			break;
		if (len < 4 ||
		    ntohs(pp->param.len) > len ||
		    ntohs(pp->param.len) < 4) {
			DEBUG(DBG_INFO, printk(" end of XRP parsing: wrong length %d\n", len));
			return;
		}
		if (pp->param.partype > XRP_CTYPE_UNKNOWN &&
		    pp->param.partype < XRP_CTYPE__COUNT) {
			if (pp->param.partype != XRP_CTYPE_NAME &&
			    ntohs(pp->param.len) !=
			    		xrp_param_len[pp->param.partype]) {
				DEBUG(DBG_INFO, printk(" wrong parameter length. class=%d, type=%d, len=%d, explen=%d\n", pp->param.parclass, pp->param.partype, ntohs(pp->param.len), pp->param.partype));
				return;
			}
			DEBUG(DBG_ALL, printk(" param class+len+expect_len: %d %d %d\n",
			pp->param.parclass, ntohs(pp->param.len), xrp_param_len[pp->param.partype]));

			switch (pp->param.parclass) {
			case XRP_CLASS_LUNAR_DSDN_RESOLVED:
				switch(pp->param.partype) {
				case XRP_CTYPE_NAME:
					if (!param2target(&s_name, pp)) {
						srcname = &s_name;
						DEBUG_TARGET(10, "DSDN resolved with", srcname);
					}
					break;
				case XRP_CTYPE_HIT128:
					if (!param2target(&s_hit, pp)) {
						srchit = &s_hit;
						DEBUG_TARGET(10, "DSDN resolved with ", srchit);
					}
					break;
				default:
					break;
				}
				break;
			default:
				break;
			}
		}
		cnt = (ntohs(pp->param.len) + 3) & ~0x3;
		len -= cnt;
		pp = (xrp_param_t*) (((char*)pp) + cnt);
	}
	
	if (srcname)
		lunar_dsdn_findname(srcname->id.name);
	else
	{
		DEBUG(DBG_LUNAR, printk("lunar_dsdn_conflict: no name given\n"));
		return;
	}
		
	/* announce new name */
	/* TODO: add random delay before sending */
	
	rnd = lunar_get_int(MIN_NAME_JITTER, MAX_NAME_JITTER);
	      
	NETBOX_Instance netbox_deltimer(&nametimer);
	NETBOX_Instance netbox_newtimer(&nametimer, 
			_VOID_VOID_ lunar_send_name, 0, rnd);	
}

void NS_CLASS lunar_dsdn_conflict(sapfpkt *p, void *aux)
{
	struct xrp_head_s *x = (struct xrp_head_s *) p->data;
	xrp_param_t *pp; // pointer to a single parameter block
	int cnt, len, rnd;
	struct lunartarget_s s_name, *srcname=0, s_hit, *srchit=0;
	int ifnr = basedev ? basedev->ifindex : 0;
	
	/* to reply */
	struct xrp_sel_eth_s *reply = 0;
	struct sockaddr sa;
	char buf[300], *cp = buf;
	struct lunartarget_s myhit;
	
	DEBUG(DBG_LUNAR, printk("DSDN: name conflict msg!\n"));
	DEBUG(DBG_LUNAR, printk("DSDN: received conflict msg (%d bytes)\n", p->len));
	
	if (p->len < sizeof(struct xrp_head_s) ||
	    ntohs(x->cmd) != XRP_CMD_LUNAR_DSDN) {
		DEBUG(DBG_INFO, printk(" wrong XRP cmd (not DSDN)\n"));
		return;
	}
	
	len = p->len - sizeof(*x);
	pp = (xrp_param_t *) (x+1);
	while (len >= 2) {
		if (ntohs(pp->param.len) & XRP_CMD_BIT)
			break;
		if (len < 4 ||
		    ntohs(pp->param.len) > len ||
		    ntohs(pp->param.len) < 4) {
			DEBUG(DBG_INFO, printk(" end of XRP parsing: wrong length %d\n", len));
			return;
		}
		if (pp->param.partype > XRP_CTYPE_UNKNOWN &&
		    pp->param.partype < XRP_CTYPE__COUNT) {
			if (pp->param.partype != XRP_CTYPE_NAME &&
			    ntohs(pp->param.len) !=
			    		xrp_param_len[pp->param.partype]) {
				DEBUG(DBG_INFO, printk(" wrong parameter length. class=%d, type=%d, len=%d, explen=%d\n", pp->param.parclass, pp->param.partype, ntohs(pp->param.len), pp->param.partype));
				return;
			}
			DEBUG(DBG_ALL, printk(" param class+len+expect_len: %d %d %d\n",
			pp->param.parclass, ntohs(pp->param.len), xrp_param_len[pp->param.partype]));

			switch (pp->param.parclass) {
			case XRP_CLASS_LUNAR_DSDN_CONFLICT:
				switch(pp->param.partype) {
				case XRP_CTYPE_NAME:
					if (!param2target(&s_name, pp)) {
						srcname = &s_name;
						DEBUG_TARGET(10, "DSDN conflict with", srcname);
					}
					break;
				case XRP_CTYPE_HIT128:
					if (!param2target(&s_hit, pp)) {
						srchit = &s_hit;
						DEBUG_TARGET(10, "DSDN conflict with ", srchit);
					}
					break;
				default:
					break;
				}
				break;
			case XRP_CLASS_LUNAR_DSDN_REPLYTO:
				if (pp->param.partype == XRP_CTYPE_SEL_ETH)
					reply = (struct xrp_sel_eth_s *) pp;
				break;
			default:
				break;
			}
		}
		cnt = (ntohs(pp->param.len) + 3) & ~0x3;
		len -= cnt;
		pp = (xrp_param_t*) (((char*)pp) + cnt);
	}
	
	if (srcname)
		lunar_dsdn_findname(srcname->id.name);
	else
	{
		DEBUG(DBG_LUNAR, printk("lunar_dsdn_conflict: no name given\n"));
		return;
	}
		
	netbox_getL3addr(&myhit, lunardev, AF_HIT);
	
	if (reply)
	{
		cp += xrp_fill_hdr(cp, XRP_CMD_LUNAR_DSDN, 0);
		cp += xrp_fill_name(cp, XRP_CLASS_LUNAR_DSDN_RESOLVED, dsdnparams);
		cp += xrp_fill_hit128(cp, XRP_CLASS_LUNAR_DSDN_RESOLVED, &myhit.id.hit);
		cp += xrp_fill_eom(cp);

		memcpy( sa.sa_data, reply->eth, ETH_ALEN);
		NETBOX_Instance netbox_l2send(ifnr, &sa, reply->sel, buf, cp - buf);
	}
	else
		DEBUG(DBG_LUNAR, printk("lunar_dsdn_conflict: no reply eth+sel given\n"));
		
	/* announce new name */
	/* TODO: add random delay before sending */
	
	rnd = lunar_get_int(MIN_NAME_JITTER, MAX_NAME_JITTER);
	      
	NETBOX_Instance netbox_deltimer(&nametimer);
	NETBOX_Instance netbox_newtimer(&nametimer, 
			_VOID_VOID_ lunar_send_name, 0, rnd);
}
				
