/* xrp.c
 *
 *   SAPF kernel module
 *
 *   Parsing and formatting support for XRP (extensible resolution
 *   protocol)
 *
 * Copyright (C) 2003 <christian.tschudin@unibas.ch>
 *
 * This file is distributed under the terms of the GNU general Public
 * License (GPL), see the file COPYING
 *
 */

#include "../include/xrp.h"
#include "../include/xrp_decl.h"


void* NS_CLASS xrp_locate(void *buf, int len, int parclass)
{
	struct xrp_common_s *cp = (struct xrp_common_s *)buf;
	int bytecnt;

	while (len >= 4) {
                if ((ntohs(cp->len) & XRP_CMD_BIT) ||
		    ntohs(cp->len) > len ||
		    ntohs(cp->len) < 4)
                        return 0;
		if (cp->parclass == parclass) {
			if (cp->partype >= XRP_CTYPE__COUNT ||
			    ntohs(cp->len) != xrp_param_len[cp->partype])
				return 0;
			return cp;
		}
		bytecnt = (ntohs(cp->len) + 3) & ~0x3;
                len -= bytecnt;
		cp = (struct xrp_common_s*) (((char*)cp) + bytecnt);
        }
	return 0;
}


int NS_CLASS xrp_fill_hdr(void *buf, int cmd, int ttl)
{
	struct xrp_head_s *x = (struct xrp_head_s *) buf;
	x->cmd = htons(0x8000 | cmd);
	x->ttl = (uint8_t)ttl;
	x->reserved = 0;
	return sizeof(*x);
}


int NS_CLASS xrp_fill_sel(void *buf, int parclass, sapfsel_t sel)
{
	struct xrp_sel_s *p = (struct xrp_sel_s *) buf;
	p->len = htons(sizeof(*p));
	p->parclass = (uint8_t)parclass;
	p->partype = XRP_CTYPE_SELECTOR;
	p->sel = sel;
	return (ntohs(p->len) + 3) & ~3;
}


int NS_CLASS xrp_fill_sel_eth(void *buf, int parclass, sapfsel_t sel, unsigned char *eth)
{
	struct xrp_sel_eth_s *p = (struct xrp_sel_eth_s *) buf;
	p->len = htons(sizeof(short) + 1 + 1 + sizeof(sapfsel_t) + ETH_ALEN);
	p->parclass = (uint8_t)parclass;
	p->partype = XRP_CTYPE_SEL_ETH;
	p->sel = sel;
	memcpy(p->eth, eth, ETH_ALEN);
	return (ntohs(p->len) + 3) & ~3;
}


int NS_CLASS xrp_fill_ipv4(void *buf, int parclass, struct in_addr ip)
{
	struct xrp_ipv4_s *p = (struct xrp_ipv4_s *) buf;
	p->len = htons(sizeof(short) + 1 + 1 + 4);
	p->parclass = (uint8_t)parclass;
	p->partype = XRP_CTYPE_IPv4;
	p->ip = ip;
	return (ntohs(p->len) + 3) & ~3;
}


#ifdef LUNAR_HAS_IPV6
int NS_CLASS xrp_fill_ipv6(void *buf, int parclass, struct in6_addr *ip6)
{
	struct xrp_ipv6_s *p = (struct xrp_ipv6_s *) buf;
	p->len = htons(sizeof(short) + 1 + 1 + sizeof(struct in6_addr));
	p->parclass = (uint8_t)parclass;
	p->partype = XRP_CTYPE_IPv6;
	memcpy(&(p->ip6), ip6, sizeof(struct in6_addr));
	return (ntohs(p->len) + 3) & ~3;
}
#endif

int NS_CLASS xrp_fill_name(void *buf, int parclass, char *s)
{
	struct xrp_name_s *p = (struct xrp_name_s *) buf;
	p->len = htons(sizeof(short) + 1 + 1 + strlen(s) + 1);
	p->parclass = (uint8_t)parclass;
	p->partype = XRP_CTYPE_NAME;
	strcpy(p->name, s);
	return (ntohs(p->len) + 3) & ~3;
}

int NS_CLASS xrp_fill_hit128(void *buf, int parclass, hit_t *hit)
{
	struct xrp_hit128_s *p = (struct xrp_hit128_s *) buf;
	p->len = htons(sizeof(short) + 1 + 1 + sizeof(p->hit));
	p->parclass = (uint8_t)parclass;
	p->partype = XRP_CTYPE_HIT128;
	memcpy(p->hit, hit, sizeof(p->hit));
	return (ntohs(p->len) + 3) & ~3;
}

int NS_CLASS xrp_fill_eom(void *buf)
{
	struct xrp_head_s *x = (struct xrp_head_s *) buf;
	x->cmd = htons(XRP_CMD_EOM);
	return sizeof(x->cmd);
}

// eof

